unit uForm1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type

  ENoMoreLines = class(Exception);

  TStringsIterator = class(TPersistent)
  private
  public

    function HasNext: boolean; virtual; abstract;
    function Next: string; virtual; abstract;

  end;

  TStringsForwardIterator = class(TStringsIterator)
  private
    FLineNo: integer;
    FList: TStrings;
  public
    constructor Create(List: TStrings);

    function HasNext: boolean; override;
    function Next: string; override;

  end;

  TStringsReverseIterator = class(TStringsIterator)
  private
    FLineNo: integer;
    FList: TStrings;
  public
    constructor Create(List: TStrings);

    function HasNext: boolean; override;
    function Next: string; override;

  end;

  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    procedure Iterate(Iterator: TStringsIterator);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

// Build a comma-delimited string from the lines in the memo
procedure TForm1.Button1Click(Sender: TObject);
var
  Iterator: TStringsIterator;
begin
  Iterator := TStringsForwardIterator.Create(Memo1.Lines);
  try
    Iterate(Iterator);
{ Uncomment the next line to see a "No More Lines" exception when you click the
  Forward button }
//    Iterator.Next;
  finally
    Iterator.Free;
  end;
end;

// Build a comma-delimited string in reverse from the lines in the memo
procedure TForm1.Button2Click(Sender: TObject);
var
  Iterator: TStringsIterator;
begin
  Iterator := TStringsReverseIterator.Create(Memo1.Lines);
  try
    Iterate(Iterator);
  finally
    Iterator.Free;
  end;
end;

procedure TForm1.Iterate(Iterator: TStringsIterator);
var
  sTemp, sMessage: string;
begin
  sMessage := '';
  while Iterator.HasNext do
  begin
    sTemp := Iterator.Next;
    sMessage := sMessage + sTemp + ', ';
  end;
  ShowMessage(sMessage);
end;

{ TStringsForwardIterator }

constructor TStringsForwardIterator.Create(List: TStrings);
begin
  inherited Create;
  FList := List;
  FLineNo := 0;
end;

function TStringsForwardIterator.HasNext: boolean;
begin
  Result := FLineNo < FList.Count;
end;

function TStringsForwardIterator.Next: string;
begin
  if HasNext then
  begin
    Result := FList.Strings[FLineNo];
    Inc(FLineNo);
  end
  else
  begin
    raise ENoMoreLines.Create('No more lines in this list');
  end;
end;

{ TStringsReverseIterator }

constructor TStringsReverseIterator.Create(List: TStrings);
begin
  inherited Create;
  FList := List;
  FLineNo := List.Count - 1;
end;

function TStringsReverseIterator.HasNext: boolean;
begin
  Result := FLineNo >= 0;
end;

function TStringsReverseIterator.Next: string;
begin
  if HasNext then
  begin
    Result := FList.Strings[FLineNo];
    Dec(FLineNo);
  end
  else
  begin
    raise ENoMoreLines.Create('No more lines in this list');
  end;
end;

end.
